@@ -21,7 +21,6 @@ import pandas as pd
 import uproot
 import uproot3
 from logzero import logger as log
-from tqdm import tqdm
 
 from . import utils
 
@@ -115,6 +114,26 @@ aliases = {
     "MC15TuneV1_ProbNNk": "probe_MC15TuneV1_ProbNNk",
     "MC15TuneV1_ProbNNp": "probe_MC15TuneV1_ProbNNp",
     "MC15TuneV1_ProbNNpi": "probe_MC15TuneV1_ProbNNpi",
+    "Brunel_MC12TuneV2_ProbNNe": "probe_Brunel_MC12TuneV2_ProbNNe",
+    "Brunel_MC12TuneV2_ProbNNghost": "probe_Brunel_MC12TuneV2_ProbNNghost",
+    "Brunel_MC12TuneV2_ProbNNk": "probe_Brunel_MC12TuneV2_ProbNNk",
+    "Brunel_MC12TuneV2_ProbNNp": "probe_Brunel_MC12TuneV2_ProbNNp",
+    "Brunel_MC12TuneV2_ProbNNpi": "probe_Brunel_MC12TuneV2_ProbNNpi",
+    "Brunel_MC12TuneV3_ProbNNe": "probe_Brunel_MC12TuneV3_ProbNNe",
+    "Brunel_MC12TuneV3_ProbNNghost": "probe_Brunel_MC12TuneV3_ProbNNghost",
+    "Brunel_MC12TuneV3_ProbNNk": "probe_Brunel_MC12TuneV3_ProbNNk",
+    "Brunel_MC12TuneV3_ProbNNp": "probe_Brunel_MC12TuneV3_ProbNNp",
+    "Brunel_MC12TuneV3_ProbNNpi": "probe_Brunel_MC12TuneV3_ProbNNpi",
+    "Brunel_MC12TuneV4_ProbNNe": "probe_Brunel_MC12TuneV4_ProbNNe",
+    "Brunel_MC12TuneV4_ProbNNghost": "probe_Brunel_MC12TuneV4_ProbNNghost",
+    "Brunel_MC12TuneV4_ProbNNk": "probe_Brunel_MC12TuneV4_ProbNNk",
+    "Brunel_MC12TuneV4_ProbNNp": "probe_Brunel_MC12TuneV4_ProbNNp",
+    "Brunel_MC12TuneV4_ProbNNpi": "probe_Brunel_MC12TuneV4_ProbNNpi",
+    "Brunel_MC15TuneV1_ProbNNe": "probe_Brunel_MC15TuneV1_ProbNNe",
+    "Brunel_MC15TuneV1_ProbNNghost": "probe_Brunel_MC15TuneV1_ProbNNghost",
+    "Brunel_MC15TuneV1_ProbNNk": "probe_Brunel_MC15TuneV1_ProbNNk",
+    "Brunel_MC15TuneV1_ProbNNp": "probe_Brunel_MC15TuneV1_ProbNNp",
+    "Brunel_MC15TuneV1_ProbNNpi": "probe_Brunel_MC15TuneV1_ProbNNpi",
     "P": "probe_P",
     "Brunel_P": "probe_Brunel_P",
     "ETA": "probe_ETA",
@@ -124,8 +143,11 @@ aliases = {
     "nSPDhits": "nSPDhits",
     "nSPDhits_Brunel": "nSPDhits_Brunel",
     "TRCHI2NDOF": "probe_TRCHI2NDOF",
+    "Brunel_TRCHI2NDOF": "probe_Brunel_TRCHI2NDOF",
     "IsMuon": "probe_isMuon",
+    "Brunel_IsMuon": "probe_Brunel_isMuon",
     "HasRich": "probe_hasRich",
+    "Brunel_HasRich": "probe_Brunel_hasRich",
 }
 
 
@@ -190,7 +212,7 @@ def get_relevant_branch_names(
                 if cut_var not in aliases:
                     log.warning(
                         (
-                            f"'Cut variable {cut_var}' is not a known alias, "
+                            f"Cut variable '{cut_var}' is not a known alias, "
                             "using raw variable"
                         )
                     )
@@ -298,69 +320,57 @@ def get_calibration_sample(
     return calibration_sample
 
 
-def root_to_dataframe(path: str, tree_name: str, branches: List[str]) -> pd.DataFrame:
+def root_to_dataframe(
+    path: str, tree_names: List[str], branches: List[str]
+) -> pd.DataFrame:
     """Return DataFrame with requested branches from tree in ROOT file.
 
     Args:
         path: Path to the ROOT file; either file system path or URL, e.g.
             root:///eos/lhcb/file.root.
-        tree_name: Name of a tree inside the ROOT file.
+        tree_names: Names of trees inside the ROOT file to read.
         branches: Branches to put in the DataFrame.
     """
-    tree = uproot.open(path)[tree_name]
+    tree = None
+    # EOS sometimes fails with a message saying the operation expired. It is
+    # intermittent and hard to replicate. See this related issue:
+    # https://github.com/scikit-hep/uproot4/issues/351. To avoid PIDCalib2
+    # completely failing in these cases, we skip the file with a warning
+    # message if this happens.
     try:
-        df = tree.arrays(branches, library="pd")  # type: ignore
-        return df
-    except uproot.exceptions.KeyInFileError as exc:  # type: ignore
-        similar_keys = utils.find_similar_strings(exc.key, list(aliases), 0.80)
-        similar_keys += utils.find_similar_strings(exc.key, tree.keys(), 0.80)
-        similar_keys += utils.find_similar_strings(
-            "probe_" + exc.key, tree.keys(), 0.80
-        )
-        # Remove duplicates while preserving ordering
-        similar_keys = list(dict.fromkeys(similar_keys))
-        log.error(
-            (
-                f"Branch '{exc.key}' not found; similar aliases and/or branches "
-                f"that exist: {similar_keys}"
+        root_file = uproot.open(path)
+    except OSError as err:
+        if "Operation expired" in err.args[0]:
+            log.error(
+                f"Failed to open '{path}' because an XRootD operation expired; skipping"
             )
-        )
-        raise
+            print(err)
+            return None  # type: ignore
+        else:
+            raise
 
+    dfs = []
+    for tree_name in tree_names:
+        tree = root_file[tree_name]
+        try:
+            dfs.append(tree.arrays(branches, library="pd"))  # type: ignore
+        except uproot.exceptions.KeyInFileError as exc:  # type: ignore
+            similar_keys = utils.find_similar_strings(exc.key, list(aliases), 0.80)
+            similar_keys += utils.find_similar_strings(exc.key, tree.keys(), 0.80)
+            similar_keys += utils.find_similar_strings(
+                "probe_" + exc.key, tree.keys(), 0.80
+            )
+            # Remove duplicates while preserving ordering
+            similar_keys = list(dict.fromkeys(similar_keys))
+            log.error(
+                (
+                    f"Branch '{exc.key}' not found; similar aliases and/or branches "
+                    f"that exist: {similar_keys}"
+                )
+            )
+            raise
 
-def calib_root_to_dataframe(
-    paths: List[str], tree_paths: List[str], branches: Dict[str, str]
-) -> pd.DataFrame:
-    """Read ROOT files via XRootD, extract branches, and save to a Pandas DF.
-
-    DataFrame columns are not named after the branches in the ROOT trees, but
-    rather by their associated 'simple user-level' analogues. E.g., 'DLLK'
-    instead of 'probe_PIDK'.
-
-    Args:
-        paths: Paths to ROOT files; either file system paths or URLs, e.g.
-            ["root:///eos/lhcb/file.root"].
-        tree_paths: Internal ROOT file paths to the relevant trees
-        branches: Dict of the branches {simple_name: branch_name} to include
-           in the DataFrame.
-
-    Returns:
-        Pandas DataFrame with the requested branches from a decay tree of a
-        single particle.
-    """
-    df_tot = pd.DataFrame()
-    for path in tqdm(paths, leave=False, desc="Reading files"):
-        for tree_path in tree_paths:
-            df = root_to_dataframe(path, tree_path, list(branches.values()))
-            df_tot = df_tot.append(df)
-
-    # Rename colums of the dataset from branch names to simple user-level
-    # names, e.g., probe_PIDK -> DLLK.
-    inverse_branch_dict = {val: key for key, val in branches.items()}
-    df_tot = df_tot.rename(columns=inverse_branch_dict)  # type: ignore
-
-    log.info(f"Read {len(paths)} files with a total of {len(df_tot.index)} events")
-    return df_tot
+    return pd.concat(dfs, ignore_index=True)  # type: ignore
 
 
 def get_tree_paths(particle: str, sample: str) -> List[str]:
